@@ -14,8 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function, unicode_literals
-
 import argparse
 import getpass
 import io
@@ -27,8 +25,7 @@ import warnings
 
 from os import environ, walk, _exit as os_exit
 from os.path import isfile, isdir, join
-from six import text_type, PY2
-from six.moves.urllib.parse import unquote, urlparse
+from urllib.parse import unquote, urlparse
 from sys import argv as sys_argv, exit, stderr, stdin
 from time import gmtime, strftime
 
@@ -78,8 +75,8 @@ Positional arguments:
                         for multiple objects.
 
 Optional arguments:
-  -a, --all             Delete all containers and objects.
-  --versions            Delete all versions
+  -a, --all             Delete all containers and objects. Implies --versions.
+  --versions            Delete all versions.
   --leave-segments      Do not delete segments of manifest objects.
   -H, --header <header:value>
                         Adds a custom request header to use for deleting
@@ -132,6 +129,8 @@ def st_delete(parser, args, output_manager, return_parser=False):
 
     (options, args) = parse_args(parser, args)
     args = args[1:]
+    if options['yes_all']:
+        options['versions'] = True
     if (not args and not options['yes_all']) or (args and options['yes_all']):
         output_manager.error('Usage: %s delete %s\n%s',
                              BASENAME, st_delete_options,
@@ -191,10 +190,6 @@ def st_delete(parser, args, output_manager, return_parser=False):
                         for o, err in r.get('result', {}).get('Errors', []):
                             # o will be of the form quote("/<cont>/<obj>")
                             o = unquote(o)
-                            if PY2:
-                                # In PY3, unquote(unicode) uses utf-8 like we
-                                # want, but PY2 uses latin-1
-                                o = o.encode('latin-1').decode('utf-8')
                             output_manager.error('Error Deleting: {0}: {1}'
                                                  .format(o[1:], err))
                             try:
@@ -529,8 +524,7 @@ Optional arguments:
 
 def st_list(parser, args, output_manager, return_parser=False):
 
-    def _print_stats(options, stats, human):
-        total_count = total_bytes = 0
+    def _print_stats(options, stats, human, totals):
         container = stats.get("container", None)
         for item in stats["listing"]:
             item_name = item.get('name')
@@ -541,17 +535,18 @@ def st_list(parser, args, output_manager, return_parser=False):
                     item_bytes = item.get('bytes')
                     byte_str = prt_bytes(item_bytes, human)
                     count = item.get('count')
-                    total_count += count
+                    totals['count'] += count
                     try:
                         meta = item.get('meta')
                         utc = gmtime(float(meta.get('x-timestamp')))
                         datestamp = strftime('%Y-%m-%d %H:%M:%S', utc)
                     except TypeError:
                         datestamp = '????-??-?? ??:??:??'
+                    storage_policy = meta.get('x-storage-policy', '???')
                     if not options['totals']:
                         output_manager.print_msg(
-                            "%5s %s %s %s", count, byte_str,
-                            datestamp, item_name)
+                            "%12s %s %s %-15s %s", count, byte_str,
+                            datestamp, storage_policy, item_name)
                 else:    # list container contents
                     subdir = item.get('subdir')
                     content_type = item.get('content_type')
@@ -576,17 +571,7 @@ def st_list(parser, args, output_manager, return_parser=False):
                             output_manager.print_msg(
                                 "%s %10s %8s %24s %s",
                                 byte_str, date, xtime, content_type, item_name)
-                total_bytes += item_bytes
-
-        # report totals
-        if options['long'] or human:
-            if not container:
-                output_manager.print_msg(
-                    "%5s %s", prt_bytes(total_count, True),
-                    prt_bytes(total_bytes, human))
-            else:
-                output_manager.print_msg(
-                    prt_bytes(total_bytes, human))
+                totals['bytes'] += item_bytes
 
     parser.add_argument(
         '-l', '--long', dest='long', action='store_true', default=False,
@@ -640,6 +625,7 @@ def st_list(parser, args, output_manager, return_parser=False):
         try:
             if not args:
                 stats_parts_gen = swift.list()
+                container = None
             else:
                 container = args[0]
                 args = args[1:]
@@ -665,12 +651,24 @@ def st_list(parser, args, output_manager, return_parser=False):
                     sort_keys=True, indent=2)
                 output_manager.print_msg('')
                 return
+
+            totals = {'count': 0, 'bytes': 0}
             for stats in stats_parts_gen:
                 if stats["success"]:
-                    _print_stats(options, stats, human)
+                    _print_stats(options, stats, human, totals)
                 else:
                     raise stats["error"]
 
+            # report totals
+            if options['long'] or human:
+                if container is None:
+                    output_manager.print_msg(
+                        "%12s %s", prt_bytes(totals['count'], True),
+                        prt_bytes(totals['bytes'], human))
+                else:
+                    output_manager.print_msg(
+                        prt_bytes(totals['bytes'], human))
+
         except SwiftError as e:
             output_manager.error(e.value)
 
@@ -1651,7 +1649,12 @@ def parse_args(parser, args, enforce_requires=True):
         return options, args
 
     if enforce_requires:
-        if options['os_auth_type'] == 'v3applicationcredential':
+        if options['os_auth_type'] and options['os_auth_type'] not in (
+                'password', 'v1password', 'v2password', 'v3password',
+                'v3applicationcredential'):
+            exit('Only "v3applicationcredential" is supported for '
+                 '--os-auth-type')
+        elif options['os_auth_type'] == 'v3applicationcredential':
             if not (options['os_application_credential_id'] and
                     options['os_application_credential_secret']):
                 exit('Auth version 3 (application credential) requires '
@@ -1659,9 +1662,6 @@ def parse_args(parser, args, enforce_requires=True):
                      'OS_APPLICATION_CREDENTIAL_SECRET to be set or '
                      'overridden with --os-application-credential-id and '
                      '--os-application-credential-secret respectively.')
-        elif options['os_auth_type']:
-            exit('Only "v3applicationcredential" is supported for '
-                 '--os-auth-type')
         elif options['auth_version'] == '3':
             if not options['auth']:
                 exit('Auth version 3 requires OS_AUTH_URL to be set or '
@@ -1728,6 +1728,9 @@ def add_default_args(parser):
     parser.add_argument('-K', '--key', dest='key',
                         default=environ.get('ST_KEY'),
                         help='Key for obtaining an auth token.')
+    parser.add_argument('-T', '--timeout', type=int, dest='timeout',
+                        default=None,
+                        help='Timeout in seconds to wait for response.')
     parser.add_argument('-R', '--retries', type=int, default=5, dest='retries',
                         help='The number of times to retry a failed '
                              'connection.')
@@ -1926,7 +1929,7 @@ def add_default_args(parser):
 def main(arguments=None):
     argv = sys_argv if arguments is None else arguments
 
-    argv = [a if isinstance(a, text_type) else a.decode('utf-8') for a in argv]
+    argv = [a if isinstance(a, str) else a.decode('utf-8') for a in argv]
 
     parser = argparse.ArgumentParser(
         add_help=False, formatter_class=HelpFormatter, usage='''
@@ -2008,7 +2011,7 @@ Examples:
 
   %(prog)s --os-auth-url https://api.example.com/v3 --auth-version 3\\
       --os-application-credential-id d78683c92f0e4f9b9b02a2e208039412 \\
-      --os-application-credential-secret APPLICTION_CREDENTIAL_SECRET \\
+      --os-application-credential-secret APPLICATION_CREDENTIAL_SECRET \\
       --os-auth-type v3applicationcredential list
 
   %(prog)s --os-auth-token 6ee5eb33efad4e45ab46806eac010566 \\
@@ -2056,8 +2059,9 @@ Examples:
         try:
             globals()['st_%s' % args[0]](parser, argv[1:], output)
         except ClientException as err:
+            trans_id = err.transaction_id
+            err.transaction_id = None  # clear it so we aren't overly noisy
             output.error(str(err))
-            trans_id = (err.http_response_headers or {}).get('X-Trans-Id')
             if trans_id:
                 output.error("Failed Transaction ID: %s",
                              parse_header_string(trans_id))
